//v-trim v-trim:textarea
//支持trim指令的html dom类型
const domType = {
    input: {
        tagName: 'INPUT',
        element: HTMLInputElement,
    },
    textarea: {
        tagName: 'TEXTAREA',
        element: HTMLTextAreaElement,
    },
}
//编辑时称发时机的事件类型
const informEventTypes = ['input', 'change']
//健盘输入时触发时机的事件类型
const diyEventType = ' free_trim_change'
//dom监听事件类型
const updateEventTypes = ['change', diyEventType]
// 获取dom
const getDom = (el, type) => {
    return el.tagName === domType[type].tagName ? el : el.querySelector(type)
}
//自定义触发事件
const dispatchEvent = (el, events) => {
    events = Array.isArray(events) ? events : [events]
    events.forEach(event => el.dispatchEvent(new CustomEvent(event)))
}
// trim处理
const trimHandle = el => {
    return event => {
        const oldValue = event.target.value
        const newVal = oldValue.trim()
        if (newVal !== oldValue) {
            event.target.value = newVal
            //通知在当前dom上的其它监听事件
            dispatchEvent(el, informEventTypes)
        }
    }
}

//对输入框等dom添加监听事件
const Trim = {
    bind: (el, binding) => {
        const type = binding.arg || 'input'
        if (!domType[type]) return
        const inputDom = getDom(el, type)
        const _trimHandle = trimHandle(el)
        el._inputDom = inputDom
        el._trimHandle = _trimHandle
        updateEventTypes.forEach(event => inputDom.addEventListener(event, el._trimHandle)) //融发一次自定义事件，防止bind制dom已经有初始值了
        setTimeout(()=>{
            dispatchEvent(el._inputDom, diyEventType)
        },0)
    },
    unbind: el => {
        updateEventTypes.forEach(event => el._inputDom && el._inputDom.removeEventListener(event, el.__trimHandle))
    },
}

//处理对输入框等dom的value直接减值的情况
const htmlDomValueAssignChange = (htmlDom) => {
    const inputProps = Object.getOwnPropertyDescriptor(htmlDom.prototype, 'value');
    Object.defineProperty(htmlDom.prototype, 'value', Object.assign({},inputProps,{
      get: inputProps.get,
      set(newVal) {
        inputProps.set.call(this, newVal);
        if (!newVal) return;
        if (newVal !== newVal.trim()) {
          dispatchEvent(this, diyEventType);
        }
      },
    }));
  };
const valueAssignChange = () => {
    Object.values(domType).forEach(val => htmlDomValueAssignChange(val.element))
}
valueAssignChange()
export default {
    install: Vue => {
        Vue.directive('trim', Trim)
    },
}
